import { event, classes, attr } from 'min-dom';
import { assign } from 'min-dash';
import 'css.escape';
import { config, edit } from 'ace-builds/src-noconflict/ace';
import 'diagram-js/lib/core';

// Enable dynamic loading of formatting modes
config.set('basePath', 'https://unpkg.com/ace-builds@1.4.12/src-noconflict');

/**
 * A code editor that allows you to add syntax highlighting and tessting to bpmn properties panel
 */
function Editor(
  config, injector, eventBus,
  canvas, elementRegistry) {

  var self = this;

  this._canvas = canvas;
  this._elementRegistry = elementRegistry;
  this._eventBus = eventBus;
  this._injector = injector;

  this._state = {
    isOpen: undefined,
    isEnabled: undefined,
  };

  this._init();

  this.toggle((config && config.open) || false);

  event.bind(this._toggle, 'click', function (event) {
    event.preventDefault();
    event.stopPropagation();

    self.toggle();
  });

  event.bind(this._run_btn, 'click', function (event) {
    event.preventDefault();
    event.stopPropagation();

    self.validate();
  });

  // Check that the currently selected bpmn task is a script task and enable or disable coding window appropriately
  eventBus.on('selection.changed', function (context) {
    if (self.isOpen()) {
      self.close();
    }
    if (context.newSelection[0] && self._parent) { // ? Try and replace self._parent with self.isOpen() cause idk why it works yet 
      if (context.newSelection[0].type == "bpmn:ScriptTask") {
        classes(self._parent).add('enabled');
        self._state.isEnabled = true;
        return;
      }
    }
    classes(self._parent).remove('enabled');
    self._state.isEnabled = false;
  });
  
  eventBus.on('editor.validate.response', function (response) {
    if(response.type == "ok"); else {
      response.msg; // Show Error Response
    }
    self._eventBus.fire("editor.toolbar.update");
  });

  eventBus.on('editor.scripts.response', function (response) {
    if(response.type == "error"){
      self.scripts = [{name:"No Scripts Available", description:"Either pound sand or contact your system administrator "}];
    } else {
      self.scripts = response.scripts;
    }
    self._eventBus.fire("editor.toolbar.update");
  });

  eventBus.on('editor.objects.response', function (response) {
    if(response.type == "error"){
      self.objects = [{name:"No Objects Available", description:"Either pound sand or contact your system administrator "}];
    } else {
      self.objects = response.objects;
    }
    self._eventBus.fire("editor.toolbar.update");
  });

  eventBus.on('editor.toolbar.update', function () {
    self._scripts_menu.innerHTML = "";
    self.scripts.forEach(element =>
      self._scripts_menu.innerHTML += `<div class="sub-trigger">${element.name}<div class="description">${element.description}</div></div>`);

    self._objects_menu.innerHTML = "";
    self.objects.forEach(element =>
      self._objects_menu.innerHTML += `<div class="sub-trigger">${element.name}</div>`);
  });
}

Editor.$inject = [
  'config.editor',
  'injector',
  'eventBus',
  'canvas',
  'elementRegistry'
];

Editor.prototype._init = function () {
  var canvas = this._canvas,
    container = canvas.getContainer();

  // create parent div
  var parent = this._parent = document.createElement('div');

  // prevent drag propagation
  event.bind(parent, 'mousedown', function (event) {
    event.stopPropagation();
  });

  classes(parent).add('djs-editor');

  container.appendChild(parent);

  // create toggle
  var toggle = this._toggle = document.createElement('div');

  classes(toggle).add('toggle');

  parent.appendChild(toggle);


  var toolbar = this._toolbar = document.createElement('div');
  classes(toolbar).add('toolbar');

  var run_btn = this._run_btn = document.createElement('div');
  run_btn.setAttribute("id", "run_btn");

  // ? I'm pretty sure I can just create a template string that will hold all these definitions but I'll have to get into that
  
  var scripts_menu_container = document.createElement('div');
  classes(scripts_menu_container).add('dropdown');

  var scripts_menu_trigger = document.createElement('div');
  scripts_menu_trigger.innerHTML = "Scripts";
  classes(scripts_menu_trigger).add('trigger');

  scripts_menu_container.appendChild(scripts_menu_trigger);
  
  var scripts_menu = this._scripts_menu = document.createElement('div');
  classes(scripts_menu).add('dropdown-content');
  self._scripts = [];

  scripts_menu_container.appendChild(scripts_menu);
  //--------------------------------------------------------//
  var objects_menu_container = document.createElement('div');
  classes(objects_menu_container).add('dropdown');

  var objects_menu_trigger = document.createElement('div');
  objects_menu_trigger.innerHTML = "Data";
  classes(objects_menu_trigger).add('trigger');

  objects_menu_container.appendChild(objects_menu_trigger);
  
  var objects_menu = this._objects_menu = document.createElement('div');
  classes(objects_menu).add('dropdown-content');
  self._objects = [];

  objects_menu_container.appendChild(objects_menu);

  toolbar.appendChild(run_btn);
  toolbar.appendChild(scripts_menu_container);
  toolbar.appendChild(objects_menu_container);

  // create ide textarea
  var ide_window = document.createElement('div');
  ide_window.setAttribute("id", "editor");
  classes(ide_window).add('ide');

  parent.appendChild(toolbar);
  parent.appendChild(ide_window);

  this._eventBus.fire('editor.toolbar.update');
};
Editor.prototype.updateObjectsMenu = function(){

};

Editor.prototype.updateScriptsMenu = function(){

};

Editor.prototype.validate = function () {
  this._eventBus.fire('editor.validate.request', { code: this._codestore.value });
};

Editor.prototype.open = function () {
  assign(this._state, { isOpen: true });

  classes(this._parent).add('open');

  var translate = this._injector.get('translate', false) || function (s) { return s; };

  attr(this._toggle, 'title', translate('Close Editor'));
  // ? There should be some way to pass the closed editor to ace
  this._ide = edit("editor");
  //this._ide.setTheme("ace/theme/monokai");
  this._ide.session.setMode("ace/mode/python");

  // grab script properties window 
  var codestore = this._codestore = document.getElementById("cam-script-val");
  // Sync code window and a properties tab
  this._ide.session.setValue(codestore.value);

  codestore.addEventListener("input", (event) => {
    this._ide.session.setValue(codestore.value);

  });
  this._ide.addEventListener("input", (event) => {
    codestore.value = this._ide.getValue();
    triggerEvent(codestore, "change");
  });

  this._eventBus.fire('editor.toggle', { open: true });

  this._eventBus.fire('editor.objects.request');
  this._eventBus.fire('editor.scripts.request');
};

Editor.prototype.close = function () {

  assign(this._state, { isOpen: false });

  classes(this._parent).remove('open');

  var translate = this._injector.get('translate', false) || function (s) { return s; };

  attr(this._toggle, 'title', translate('Open Editor'));

  this._eventBus.fire('editor.toggle', { open: false });
};

Editor.prototype.toggle = function (open) {

  var currentOpen = this.isOpen();

  if (typeof open === 'undefined') {
    open = !currentOpen;
  }

  if (open == currentOpen) {
    return;
  }

  if (open) {
    this.open();
  } else {
    this.close();
  }
};

Editor.prototype.isOpen = function () {
  return this._state.isOpen;
};

/**
 * Triggers a change event
 *
 * @param element on which the change should be triggered
 * @param eventType type of the event (e.g. click, change, ...)
 */
var triggerEvent = function (element, eventType) {

  var evt;

  eventType = eventType || 'change';

  try {

    // Chrome, Safari, Firefox
    evt = new MouseEvent((eventType), {
      view: window,
      bubbles: true,
      cancelable: true
    });
  } catch (e) {

    // IE 11, PhantomJS (wat!)
    evt = document.createEvent('MouseEvent');

    evt.initEvent((eventType), true, true);
  }

  return element.dispatchEvent(evt);
};

var index = {
  // __depends__: [ CoreModule ], // {2}
  __init__: [ 'editor' ], // {3}
  editor: [ 'type', Editor ] // {1}
};

export default index;
